#!/bin/bash

#################################
# BDraw v1.8 by SFR'2012-2015   #
# GNU GPL v2 applies            #
#################################

# -------------
# Requirements:
# -------------
# Bash >= 3.0.0
# Gktdialog >= 0.8.0 (for <button>)
# Gtkdialog >= 0.8.2 (for <pixmap>)
# Xdialog >= 2.3.1
# rsvg-convert
# xwininfo
# getcurpos
# base64
# awk, sed, head, tail, tr, grep, wc, bc, dc, seq (etc.)

# =============================================================================
# Init
# =============================================================================

INTERVAL=333		# refresh speed (ms)

export APPNAME="BDraw v1.8"
MYNAME="$(readlink -e "$0")"
sleep 0.3
if [ `busybox pidof "$(basename "$MYNAME")" -o %PPID | wc -w` -gt 1 ]; then
 Xdialog --centre --title "$APPNAME" --infobox "BDraw is already running..." 0 0 10000 &
 exit 1
fi

func_vercmp_gt () {
  [ "${1}" != "$(echo -e "${1}\n${2}" | sort -t '.' -k1,1n -k2,2n -k3,3n -k4,4n -k5,5n | head -n1)" ]
}

[ `which gtkdialog4` ] && export GTKDIALOG=gtkdialog4 || export GTKDIALOG=gtkdialog
GTKVER=( $($GTKDIALOG -v) ); GTKVER=${GTKVER[2]}
#vercmp $GTKVER lt 0.8.0 && Xdialog --centre --title "$APPNAME" --infobox "Gtkdialog >= 0.8.0 required!\nExiting..." 0 0 10000 && exit
#vercmp $GTKVER lt 0.8.2 && BUT_VAR=true || BUT_VAR=false
func_vercmp_gt 0.8.0 $GTKVER && Xdialog --centre --title "$APPNAME" --infobox "Gtkdialog >= 0.8.0 required!\nExiting..." 0 0 10000 && exit
func_vercmp_gt 0.8.2 $GTKVER && BUT_VAR=true || BUT_VAR=false

#export TEMPDIR=/tmp/BDraw_temp_dir_$$
export TEMPDIR=/dev/shm/BDraw_temp_dir_$$
trap 'rm -rf $TEMPDIR' EXIT #INT TERM 

export PICRES=$TEMPDIR/picres
export CURCOR=$TEMPDIR/curcor
export PIC=$TEMPDIR/pic.svg
export PIC_TEMP=$TEMPDIR/pic_temp.svg
export COORDINATES=$TEMPDIR/coordinates
export DASH_PIC=$TEMPDIR/dash.svg
export REDO=$TEMPDIR/redo
export REDO_TEMP=$TEMPDIR/redo_temp
export LOADSTATUS=$TEMPDIR/loadstatus
export EXITSTATUS=$TEMPDIR/exitstatus
export FREE_STORE=$TEMPDIR/freestore
export FREE_STATUS=$TEMPDIR/freestatus
export BUT_STATE=$TEMPDIR/butstate
export C2PNG=$TEMPDIR/convert2png
export PREV=$TEMPDIR/preview
export UNDO_MARK=$TEMPDIR/undo_mark
export PREV_ARC=$TEMPDIR/prev_arc
export FRAME_PIC_X=$TEMPDIR/frame_x.svg
export FRAME_PIC_Y=$TEMPDIR/frame_y.svg

[ -d $TEMPDIR ] && rm -rf $TEMPDIR
mkdir $TEMPDIR

# -----------------------------------------------------------------------------

echo " Start: ---,---   End: ---,---" > $CURCOR
echo -n > $REDO
echo -n > $LOADSTATUS
echo 0 > $BUT_STATE
echo false > $PREV
echo -n > $COORDINATES
echo -n > $PREV_ARC

# -----------------------------------------------------------------------------

# =============================================================================
# FUNCTIONS
# =============================================================================

# ------------------
# Make empty picture
# ------------------
make_empty () {
echo -n '<svg width="'$PIC_X'" height="'$PIC_Y'" xmlns="http://www.w3.org/2000/svg">
</svg>' > $PIC
}
export -f make_empty

# ------------------------------------------------------------------
# Make pictures for horizonal & vertical frames (<pixmap> mode only)
# ------------------------------------------------------------------
make_frames () {
echo -n '<svg width="'$PIC_X'" height="1" xmlns="http://www.w3.org/2000/svg">
<line x1="0" y1="0" x2="'$PIC_X'" y2="0" stroke="black" stroke-dasharray="1,1" />
</svg>' > $FRAME_PIC_X
echo -n '<svg width="1" height="'$PIC_Y'" xmlns="http://www.w3.org/2000/svg">
<line x1="0" y1="0" x2="0" y2="'$PIC_Y'" stroke="black" stroke-dasharray="1,1" />
</svg>' > $FRAME_PIC_Y
}
export -f make_frames

# ----------------------------
# Make picture for dash button
# ----------------------------
make_dash_button () {
! $DASH && DA="0,0" || DA="3,3"
echo -n '<svg xmlns="http://www.w3.org/2000/svg">
<line x1="0" y1="3" x2="33" y2="3" stroke-dasharray="'$DA'" stroke="black" stroke-width="3"/>
</svg>' > $DASH_PIC
}
export -f make_dash_button && DASH=false && make_dash_button

# ----------------------------------------
# Get current, relative position of cursor
# ----------------------------------------
get_xy () {
  read MX MY WX WY WI <<< `getcurpos && xwininfo -name "$APPNAME" 2>/dev/null | awk 'NR>=4&&NR<=5 {print $4} NR==8 {print $2}'`
  X=$((($MX-$WX)-($WI-$PIC_X)/2))
  Y=$(($MY-$WY-$YF))
}
export -f get_xy

# --------------------------------
# Store coordinates for "Freehand"
# --------------------------------
store_free_hand () {
  while $(<$FREE_STATUS); do
    read MX MY <<< `getcurpos`
    X=$((($MX-$WX)-($WI-$PIC_X)/2)); Y=$(($MY-$WY-$YF))
    [ "$X $Y" != "`tail -n1 $FREE_STORE`" ] && echo $X $Y >> $FREE_STORE
    sleep 0.05
  done
  tail -n -1 $FREE_STORE > $COORDINATES && head -n 1 $FREE_STORE >> $COORDINATES
}
export -f store_free_hand

# -----------------------------------------------
# This function is called on "button-press-event"
# -----------------------------------------------
# Store start X & Y
# (or store continuously for "Freehand")
# (or draw "Arc")
# -----------------------------------------------
store_xy () {
  get_xy
  echo 1 > $BUT_STATE
  # If "Freehand" - start storing coordinates
  if [[ "$MODE" = Freehand ]]; then
    echo $X $Y > $FREE_STORE
    echo true > $FREE_STATUS
    store_free_hand &
    return
  fi
  
  if [ $BUTTON != 3 ]; then
    echo $X $Y > $COORDINATES
  elif [ $BUTTON = 3 ] && [[ "$MODE" = Arc ]]; then
    (( $X<0 || $X>=$PIC_X || $Y<0 || $Y>=$PIC_Y )) && return 
    draw_arc
  fi
}
export -f store_xy

# --------------------------------
# Update current coordinates' view
# --------------------------------
update_coordinates_view () {
  get_xy
  (( $X<0 || $X>=$PIC_X || $Y<0 || $Y>=$PIC_Y )) && X="---" && Y="---"
  BS=$(<$BUT_STATE)
  if [ $BS = 0 ] || ([ "$MODE" = Arc ] && [ $BS = 2 ] && [[ $(<$PREV_ARC) = false ]]); then
      echo " Start: $X,$Y   End: $X,$Y" > $CURCOR
  else
    if [ "$MODE" = Freehand ]; then
      read SX SY <<< `head -n 1 $FREE_STORE`
    else
      read SX SY NUL NUL < $COORDINATES
    fi
    echo " Start: $SX,$SY   End: $X,$Y" > $CURCOR
  fi
}
export -f update_coordinates_view

# ---------------
# Undo last shape
# ---------------
undo () {
  LINES=`wc -l $PIC | awk '{print $1}'`
  [ "$LINES" -lt 3 ] && return
  awk 'NR<='$(($LINES-2))'' $PIC > $PIC_TEMP
  echo '</svg>' >> $PIC_TEMP
  awk 'NR=='$(($LINES-1))'' $PIC >> $REDO
  
  mv $PIC_TEMP $PIC
}
export -f undo

# -------------------------
# Undo for Preview (faster)
# -------------------------
undo4prev () {
  $(<$PREV) && $(<$UNDO_MARK) || return
  { head -n -2 $PIC; echo '</svg>'; } > $PIC_TEMP
  mv $PIC_TEMP $PIC
  echo false > $PREV
}
export -f undo4prev

# ---------------
# Redo last shape
# ---------------
redo () {
  LINES_REDO=`wc -l $REDO | awk '{print $1}'`
  LINES_PIC=`wc -l $PIC | awk '{print $1}'`
  [ "$LINES_REDO" -lt 1 ] && return
  awk 'NR<='$(($LINES_PIC-1))'' $PIC > $PIC_TEMP
  awk 'NR=='$LINES_REDO'' $REDO >> $PIC_TEMP
  echo '</svg>' >> $PIC_TEMP
  awk 'NR<='$(($LINES_REDO-1))'' $REDO > $REDO_TEMP
  
  mv $PIC_TEMP $PIC
  mv $REDO_TEMP $REDO
}
export -f redo

# --------
# Freehand
# --------
draw_free_hand () {
  POINTS=$(wc -l $FREE_STORE | awk '{print $1}')
  [ $POINTS -lt 2 ] && return

  for i in `seq 2 2 $POINTS`; do
    read SX SY EX EY <<< `awk 'NR>='$i' && NR<='$(($i+1))'' $FREE_STORE`
    [ "$EX" = "" ] && EX=$SX && EY=$SY
    MX=$((($SX+$EX)/2))
    MY=$((($SY+$EY)/2))
    FREE_PATH=$FREE_PATH" S $SX $SY $MX $MY"
  done

  FREE_PATH="`head -n 1 $FREE_STORE` "$FREE_PATH
  echo '<path d="M '$FREE_PATH'" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" stroke-width="'$SW'" fill="none" opacity="'$OPAC'"/>' >> $PIC_TEMP
}
export -f draw_free_hand

# -----------------------------
# Freehand for preview (faster)
# -----------------------------
draw_free_hand_preview () {
  FREE_PATH="`awk -F'\n' '{print $1; print " L "}' $FREE_STORE | tr -d '\n'`"
  echo '<path d="M '$FREE_PATH'" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" stroke-width="'$SW'" fill="none" opacity="'$OPAC'"/>' >> $PIC_TEMP
}
export -f draw_free_hand_preview

# ----
# Line
# ----
draw_line () {
  echo '<line x1="'$SX'" y1="'$SY'" x2="'$X'" y2="'$Y'" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" stroke-width="'$SW'" opacity="'$OPAC'"/>' >> $PIC_TEMP
}
export -f draw_line

# ---
# Arc
# ---
draw_arc () {
  read SX SY EX EY <<< `cat $COORDINATES`
  [ ! $EX ] && return
  OPAC=${OPAC/,/.}
  D1=$(($SX-$EX)); D1=${D1/-/}
  [ $SX -ge $EX ] && DX=$(($X-$SX+($D1/2))) || DX=$(($X-$EX+($D1/2)))
  D2=$(($SY-$EY)); D2=${D2/-/}
  [ $SY -ge $EY ] && DY=$(($Y-$SY+($D2/2))) || DY=$(($Y-$EY+($D2/2)))

  $DASH && DA="$(($SW*2))"",""$(($SW*2))" || DA="0,0"
  
  head -n -1 $PIC > $PIC_TEMP

echo '<path d="M '$SX' '$SY' Q '$(($X+$DX))' '$(($Y+$DY))' '$EX' '$EY'" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" fill="none" stroke-width="'$SW'" opacity="'$OPAC'"/>
</svg>' >> $PIC_TEMP

  mv $PIC_TEMP $PIC
  echo -n > $REDO
}
export -f draw_arc

# ---------
# Rectangle
# ---------
draw_rectangle () {
  echo '<rect x="'$SX'" y="'$SY'" width="'$(($X-$SX))'" height="'$(($Y-$SY))'" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" fill="'$FILL'" stroke-width="'$SW'" opacity="'$OPAC'"/>' >> $PIC_TEMP
}
export -f draw_rectangle

# -------------------------------------------
# Marker Rectangle (for Import/Export Clipart
# -------------------------------------------
draw_marker_rectangle () {
  D1=$(($X-$SX)); D2=$(($Y-$SY))
  O="0.5"; SW=1; DA="2,2"; STROKE_COL="#000000"
  echo '<pattern id="Pattern" patternUnits="userSpaceOnUse" x="0" y="0" width="10" height="10"><rect x="0" y="0" width="5" height="5" fill="lightgray" /><rect x="5" y="5" width="5" height="5" fill="lightgray" /></pattern><rect x="'$SX'" y="'$SY'" width="'$D1'" height="'$D2'" fill="url(#Pattern)" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" stroke-width="'$SW'" opacity="'$O'" />' >> $PIC_TEMP
}
export -f draw_marker_rectangle

# ------
# Circle
# ------
draw_circle () {
  D1=$(($SX-$X)); D2=$(($SY-$Y))
  R=`bc <<< 'sqrt('$D1'^2+'$D2'^2)'`
  echo '<circle cx="'$SX'" cy="'$SY'" r="'$R'" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" fill="'$FILL'" stroke-width="'$SW'" opacity="'$OPAC'"/>' >> $PIC_TEMP
}
export -f draw_circle

# -------
# Ellipse
# -------
draw_ellipse () {
  D1=$(($SX-$X)); D1=${D1/-/}; D2=$(($SY-$Y)); D2=${D2/-/}
  echo '<ellipse cx="'$SX'" cy="'$SY'" rx="'$D1'" ry="'$D2'" stroke-dasharray="'$DA'" stroke-linecap="round" stroke="'$STROKE'" fill="'$FILL'" stroke-width="'$SW'" opacity="'$OPAC'"/>' >> $PIC_TEMP
}
export -f draw_ellipse

# ----
# Text
# ----
draw_text () {
  STATUS=`Xdialog --stdout --title "$APPNAME" --backtitle "Choose font:" --fontsel "" 0 0 2>/dev/null`
  [ "$?" -ne 0 ] && return

  SIZE=`echo "$STATUS" | rev | awk '{print $1}' | rev`
  STYLE="`echo $STATUS | grep -o Italic | tr '[:upper:]' '[:lower:]'`"
  WEIGHT="`echo $STATUS | grep -o Bold | tr '[:upper:]' '[:lower:]'`"
  NAME="`echo $STATUS | sed 's/Bold //;s/Italic //' | rev | cut -f2- -d ' ' | rev`"

  TEXT=`Xdialog --stdout --title "$APPNAME" --inputbox "Enter text:" 0 0 2>/dev/null`
  [ "$?" -ne 0 ] && return

  TEXT="`echo "$TEXT" | sed -e 's~&~\&amp;~g' -e 's~<~\&lt;~g'  -e  's~>~\&gt;~g'`"

  echo '<text x="'$SX'" y="'$(($SY))'" xml:space="preserve" font-family="'$NAME'" font-style="'$STYLE'" font-weight="'$WEIGHT'" font-size="'$SIZE'" fill="'$STROKE'" opacity="'$OPAC'">'"$TEXT"'</text>' >> $PIC_TEMP
}
export -f draw_text

# ---------------------
# Import external image
# ---------------------
import_image () {
  [ "$SX" = "$X" ] || [ "$SY" = "$Y" ] && return

  FILENAME="`Xdialog --stdout --title "$APPNAME" --backtitle "Select file to import (.bdr .png .svg .bmp .gif .jpg)" --fselect "" 0 0 2>/dev/null`"
  [ "$?" -ne 0 ] && return

  FILENAME_TEMP="`readlink -e "$FILENAME"`"
  FILENAME_TEMP="`echo "$FILENAME_TEMP" | tr '[:upper:]' '[:lower:]'`"

  case "${FILENAME_TEMP##*.}" in
    "svg"|"bdr" )
      X_RES=`cat "$FILENAME" | tr -d ' \t' | grep -m1 -io "width=".[0-9]* | tr -cd '[[:digit:]]'`
      Y_RES=`cat "$FILENAME" | tr -d ' \t' | grep -m1 -io "height=".[0-9]* | tr -cd '[[:digit:]]'` 
      rsvg-convert -f png -w $X_RES -h $Y_RES "$FILENAME" > $C2PNG
      FILENAME=$C2PNG
    ;;
    "png"|"bmp"|"gif"|"jpg"|"jpeg" )
      break
    ;;
    * )
      Xdialog --timeout 10 --title "$APPNAME" --msgbox "Supported formats are:\n.bdr .png .svg .bmp .gif .jpg" 0 0
      return
    ;;
  esac

  EXT="${FILENAME_TEMP##*.}"
  [ "$EXT" = "jpg" ] && EXT="jpeg"
  [ "$EXT" = "bdr" ] || [ "$EXT" = "svg" ] && EXT="png"

  D1=$(($X-$SX))
  D2=$(($Y-$SY))

  echo -n '<image x="'$SX'" y="'$SY'" width="'$D1'" height="'$D2'" preserveAspectRatio="xmidYmid" xlink:href="data:image/'$EXT';base64,' >> $PIC_TEMP
  echo -n "`base64 "$FILENAME"`" | tr -d '\n' >> $PIC_TEMP
  echo '" opacity="'$OPAC'"/>' >> $PIC_TEMP
}
export -f import_image

# ==============
# Export clipart
# ==============
export_image () {
  [ "$SX" = "$X" ] || [ "$SY" = "$Y" ] && return
  FILENAME="`Xdialog --stdout --title "$APPNAME" --backtitle "Save clipart as:" --fselect "" 0 0 2>/dev/null`"
  [ "$?" -ne 0 ] && return
  [ "${FILENAME##*.}" != "png" ] && FILENAME="$FILENAME"".png"
  if [ -f "$FILENAME" ]; then
    Xdialog --title "$APPNAME" --yesno "File exists! Overwrite?" 0 0
    [ "$?" -ne 0 ] && return
  fi

  [ "$SX" -gt "$X" ] && Z=$SX && SX=$X && X=$Z
  [ "$SY" -gt "$Y" ] && Z=$SY && SY=$Y && Y=$Z

  D1=$(($X-$SX))
  D2=$(($Y-$SY))

  echo '<svg viewBox="'$SX $SY $D1 $D2'" width="'$D1'" height="'$D2'" xmlns="http://www.w3.org/2000/svg">' > $C2PNG
  tail -n +2 $PIC >> $C2PNG

  rsvg-convert -f png -w $D1 -h $D2 $C2PNG > "$FILENAME"
}
export -f export_image

# -------------------------
# Draw line for Arc Preview
# -------------------------
draw_line_for_arc () {
  head -n -1 $PIC > $PIC_TEMP

  read SX SY NUL NUL < $COORDINATES
  echo -e '<line x1="'$SX'" y1="'$SY'" x2="'$X'" y2="'$Y'" stroke-dasharray="4,4" stroke-linecap="round" stroke="'$STROKE'" stroke-width="2" opacity="0.5"/>\n</svg>' >> $PIC_TEMP
  mv $PIC_TEMP $PIC
  echo -n > $REDO
}
export -f draw_line_for_arc

# -------------------------------------------------
# This function is called on "button-release-event"
# -------------------------------------------------
# Draw a shape
# -------------------------------------------------
draw () {
  echo true > $UNDO_MARK
  PREV_V=$(<$PREV)
  
  [ $PREV_V != true ] && echo false > $FREE_STATUS  # Stop storing XYs for "Freehand"
  
  get_xy
  
  [ "$MODE" = Arc ] && [ $PREV_V = true ] && [[ $(<$PREV_ARC) = true ]] && draw_line_for_arc && return
  [ "$MODE" = Arc ] && [ $PREV_V = true ] && [[ $(<$PREV_ARC) = false ]] && draw_arc && return

  echo 0 > $BUT_STATE

  (( $X<0 || $X>=$PIC_X || $Y<0 || $Y>=$PIC_Y )) && echo false > $UNDO_MARK && return

  [ "$MODE" = Arc ] && [ $BUTTON != 3 ] && echo $X $Y >> $COORDINATES && return
  [ "$MODE" = Arc ] && return

  head -n -1 $PIC > $PIC_TEMP

  $DASH && DA="$(($SW*2)),$(($SW*2))" || DA="0,0"

  read SX SY NUL NUL < $COORDINATES
  OPAC=${OPAC/,/.}
  
  case "$MODE" in
    "Freehand" ) [ $BUTTON ] && draw_free_hand || draw_free_hand_preview;;
    "Line" ) draw_line;;
    "Rectangle" ) FILL="none"; draw_rectangle;;
    "Rectangle (filled)") draw_rectangle;;
    "Circle" ) FILL="none"; draw_circle;;
    "Circle (filled)" ) draw_circle;;
    "Ellipse" ) FILL="none"; draw_ellipse;;
    "Ellipse (filled)" ) draw_ellipse;;
    "Text" ) draw_text;;
    "Import Clipart" ) [ ! $BUTTON ] && draw_marker_rectangle || import_image ;;
    "Export Clipart" ) [ ! $BUTTON ] && draw_marker_rectangle || export_image ;;
  esac
  
  echo '</svg>' >> $PIC_TEMP
  mv $PIC_TEMP $PIC
  echo -n > $REDO
}
export -f draw

# ------------
# Load Picture
# ------------
load_pic () {
  [ "$FILENAME" = "" ] && return
  [ "${FILENAME##*.}" != "bdr" ] && return
  cp -f "$FILENAME" $PIC
  PIC_X=`head -1 $PIC | awk '{print $2}' | tr -cd '[:digit:]'`
  PIC_Y=`head -1 $PIC | awk '{print $3}' | tr -cd '[:digit:]'`
  echo $PIC_X $PIC_Y > $PICRES
  echo "GOTCHA" > $LOADSTATUS
}
export -f load_pic

# ------------
# Save Picture
# ------------
save_pic () {

  if [ "$FILENAME" = "" ]; then
    Xdialog --timeout 5 --title "$APPNAME" --msgbox "Empty filename, nothing saved." 0 0
    return
  fi

  if [ "${FILENAME##*.}" != "bdr" ] && [ "${FILENAME##*.}" != "png" ]; then
    EXT="`Xdialog --stdout --title "$APPNAME" --radiolist "Select format:" 0 0 2 Internal .bdr true External .png false 2>/dev/null`"
    [ "$?" -ne 0 ] && return
    [ "$EXT" = "Internal" ] && FILENAME="$FILENAME"".bdr" || FILENAME="$FILENAME"".png"
  fi

  if [ "${FILENAME##*.}" = "png" ]; then
    RESOLUTION="`Xdialog --stdout --title "$APPNAME" --2spinsbox "Choose output resolution:" 0 0 1 4096 $PIC_X "Width" 1 4096 $PIC_Y "Height" 2>/dev/null`"
    [ "$?" -ne 0 ] && return
    O_X="`echo $RESOLUTION | cut -f1 -d '/'`"
    O_Y="`echo $RESOLUTION | cut -f2 -d '/'`"
  fi

  if [ -f "$FILENAME" ]; then
    Xdialog --title "$APPNAME" --yesno "File exists! Overwrite?" 0 0
    [ "$?" -ne 0 ] && return
  fi

  if [ "${FILENAME##*.}" = "png" ]; then
    rsvg-convert -f png -w $O_X -h $O_Y "$PIC" > "$FILENAME"
  else
    cp -f $PIC "$FILENAME"
  fi
}
export -f save_pic

# ---------------------------
# Enable/disable preview mode
# ---------------------------
check_button () {
  get_xy
  (( $X<0 || $X>=$PIC_X || $Y<0 || $Y>=$PIC_Y )) && echo false > $PREV && return
  
  BS=$(<$BUT_STATE)
  case "$MODE" in
    Text )
      echo false > $PREV && return
    ;;
    Arc )
      COOR=`wc -l $COORDINATES | awk '{print $1}'`
      echo false > $PREV_ARC
      [ $COOR -gt 1 ] && echo true > $PREV && return
      [ $COOR -lt 1 ] && echo false > $PREV && return
      [ $BS != 0 ] && echo true > $PREV && echo true > $PREV_ARC
    ;;
    * )
      (( ! $BS )) && echo false > $FREE_STATUS && echo false > $PREV && return
      echo true > $PREV
    ;;
  esac
}
export -f check_button

# -----------------------------------------------------------------------------

# =============================================================================
# GTKDIALOG STRUCTURES (1)
# =============================================================================

export LOAD='
<window title="'$APPNAME'" icon-name="gtk-open">
  <frame Filename:>
    <hbox>
      <entry fs-action="file" fs-folder="'$HOME'" fs-filters="*.bdr">
        <variable>FILENAME</variable>
      </entry>
      <button>
        <variable>FILESELECT</variable>
        <input file stock="gtk-open"></input>
        <action type="fileselect">FILENAME</action>
      </button>
    </hbox>
    <hbox>
      <button ok>
        <action>load_pic</action>
        <action>exit:load</action>
      </button>
      <button cancel>
        <action>exit:abort</action>
      </button>
    </hbox>   
  </frame>
<action signal="hide">exit:abort</action>
</window>
'

# -----------------------------------------------------------------------------

export INITIAL='
<window title="'$APPNAME'" allow-grow="false">
<frame>
    <vbox>
      <text use-markup="true"><label>"<span weight='"'bold'"' size='"'x-large'"'><b><big>'$APPNAME'</big></b></span>"</label></text>
      <hseparator></hseparator>
      <button>
        <label>New</label>
        <input file stock="gtk-new"></input>
        <action>exit:NEW</action>
      </button>
      <button>
        <label>Open</label>
        <input file stock="gtk-open"></input>
        <action>exit:OPEN</action>
      </button>
      <button>
        <label>Quit</label>
        <input file stock="gtk-quit"></input>
        <action>exit:abort</action>
      </button>
    </vbox>
  </frame>
<action signal="hide">exit:abort</action>
</window>
'

# -----------------------------------------------------------------------------

# =============================================================================
# START
# =============================================================================

cd $HOME

# --------------
# Initial window
# --------------
I=$IFS; IFS=""
for STATEMENTS in  $($GTKDIALOG -cp INITIAL); do
  eval $STATEMENTS
done
IFS=$I

[ "$BUT_VAR" = "true" ] && export YF=9 || export YF=6

if [ "$EXIT" = "abort" ]; then
  exit 0
elif [ "$EXIT" = "NEW" ]; then
  DEFX=512
  DEFY=256
  MAXX=2048
  MAXY=2048
  
  RESOLUTION="`Xdialog --stdout --title "$APPNAME" --2spinsbox "Choose picture resolution:" 0 0 1 $MAXX $DEFX "Width" 1 $MAXY $DEFY "Height" 2>/dev/null`"
  [ "$?" -ne 0 ] && exec $MYNAME
  PIC_X="`echo $RESOLUTION | cut -f1 -d '/'`"
  PIC_Y="`echo $RESOLUTION | cut -f2 -d '/'`"
  echo $PIC_X $PIC_Y > $PICRES
  make_empty
elif [ "$EXIT" = "OPEN" ]; then
  $GTKDIALOG -cp LOAD
  [ "`cat $LOADSTATUS`" != "GOTCHA" ] && exec $MYNAME
fi

read PIC_X PIC_Y < $PICRES
export PIC_X
export PIC_Y
make_frames

# -----------------------------------------------------------------------------
# GTKDIALOG STRUCTURES (2)
# -----------------------------------------------------------------------------
export SAVE='
<window title="'$APPNAME'" icon-name="gtk-save-as" modal="true">
  <frame Filename:>
    <hbox>
      <entry fs-action="newfile" fs-folder="'$HOME'" fs-filters="'*.bdr'|'*.png'">
        <variable>FILENAME</variable>
      </entry>
      <button>
        <variable>FILESELECT</variable>
        <input file stock="gtk-open"></input>
        <action type="fileselect">FILENAME</action>
      </button>
    </hbox>
    <hbox>
      <button ok>
        <action>save_pic</action>
        <action type="closewindow">FILENAME</action>
      </button>
      <button cancel>
        <action type="closewindow">FILENAME</action>
      </button>
    </hbox>
  </frame>
</window>
'

# -----------------------------------------------------------------------------

export SURE='
<window title="'$APPNAME'" modal="true" icon-name="gtk-quit" resizable="false">
  <vbox>
    <hbox>
      <pixmap>
        <variable>SUREWINDOW</variable>
        <input file stock="gtk-dialog-warning"></input>
      </pixmap>
      <vbox>
        <text><label>"Are you sure you want to exit?"</label></text>
        <text><label>"All unsaved data will be lost!"</label></text>
      </vbox>
    </hbox>
    <hbox>
      <button no>
        <action type="closewindow">SUREWINDOW</action>
      </button>
      <button yes>
        <action>echo -n > $EXITSTATUS</action>
        <action type="exit">Exit</action>
      </button>
    </hbox>
  </vbox>
</window>
'

# ------------------
# Export MAIN dialog
# ------------------
if [ "$BUT_VAR" = "true" ]; then
SWITCH='
<button relief="2">
  <variable>PICTURE</variable>
  <input file>"'$PIC'"</input>
  <action signal="button-press-event">store_xy</action>
  <action signal="button-press-event">refresh:PICTURE</action>
  <action signal="button-release-event">draw</action>
  <action signal="button-release-event">refresh:PICTURE</action>
  <action signal="leave-notify-event">echo false > $PREV</action>
  <action signal="leave-notify-event">refresh:PICTURE</action>
</button>
'
else
SWITCH='
<eventbox above-child="false" visible-window="false">
  <hbox spacing="0">
    <pixmap><input file>"'$FRAME_PIC_Y'"</input></pixmap>
    <vbox spacing="0">
      <pixmap><input file>"'$FRAME_PIC_X'"</input></pixmap>
      <pixmap>
        <variable>PICTURE</variable>
        <input file>"'$PIC'"</input>
      </pixmap>
      <pixmap><input file>"'$FRAME_PIC_X'"</input></pixmap>
    </vbox>
    <pixmap><input file>"'$FRAME_PIC_Y'"</input></pixmap>
  </hbox>
  <action signal="button-press-event">store_xy</action>
  <action signal="button-press-event">refresh:PICTURE</action>
  <action signal="button-release-event">draw</action>
  <action signal="button-release-event">refresh:PICTURE</action>
  <action signal="leave-notify-event">echo false > $PREV</action>
  <action signal="leave-notify-event">refresh:PICTURE</action>
</eventbox>
'
fi

export MAIN='
<window title="'$APPNAME'" resizable="false">
  <vbox>
    <hbox homogeneous="true">
      '$SWITCH'
    </hbox>
    <frame Toolbox:>
      <vbox>
        <hbox space-fill="true" space-expand="true">
          <comboboxtext>
            <variable>MODE</variable>
            <default>Freehand</default>
            <item>Freehand</item>
            <item>Line</item>
            <item>Arc</item>
            <item>Text</item>
            <item>Rectangle</item>
            <item>Rectangle (filled)</item>
            <item>Circle</item>
            <item>Circle (filled)</item>
            <item>Ellipse</item>
            <item>Ellipse (filled)</item>
            <item>Import Clipart</item>
            <item>Export Clipart</item>
            <action>echo false > $PREV</action>
            <action>echo 0 > $BUT_STATE</action>
            <action>refresh:PICTURE</action>
          </comboboxtext>

          <spinbutton range-min="0" range-max="100" range-step="1" range-value="1">
            <variable>SW</variable>
          </spinbutton>

          <spinbutton range-min="0" range-max="1" range-step="'`busybox dc 1 100 / p`'" range-value="1">
            <variable>OPAC</variable>
          </spinbutton>

          <colorbutton>
            <variable>STROKE</variable>
            <default>#000000</default>
          </colorbutton>

          <colorbutton>
            <variable>FILL</variable>
            <default>#ffffff</default>
          </colorbutton>

          <togglebutton>
            <variable>DASH</variable>
            <default>false</default>
            <input file>"'$DASH_PIC'"</input>
            <action>make_dash_button</action>
            <action>refresh:DASH</action>
          </togglebutton>          
        </hbox>
        
        <hbox space-fill="true" space-expand="true">  
          <button>
            <label>Undo</label>
            <input file stock="gtk-undo"></input>
            <action>undo</action>
            <action>refresh:PICTURE</action>
          </button>

          <button>
            <label>Redo</label>
            <input file stock="gtk-redo"></input>
            <action>redo</action>
            <action>refresh:PICTURE</action>
          </button>

          <button>
            <label>Save</label>
            <input file stock="gtk-save-as"></input>
            <action>launch:SAVE</action>
          </button>

          <button>
            <label>Close</label>
            <input file stock="gtk-close"></input>
            <action>launch:SURE</action>
         </button>
        </hbox>
      </vbox>
    </frame>
    
    <hbox homogeneous="true">
      <statusbar has-resize-grip="false" space-fill="true">
        <variable>CUR_DISPLAY</variable>
        <input file>'$CURCOR'</input>
      </statusbar>
      <statusbar has-resize-grip="false" space-fill="true">
        <variable>RES_DISPLAY</variable>
        <input>echo -n " Width: " && cat $PICRES | cut -f1 -d " " | tr -d "\n" && echo -n "   Height: " && cat $PICRES | cut -f2 -d " " | tr -d "\n"</input>
      </statusbar>

      <timer milliseconds="true" interval="'$INTERVAL'" visible="false">
        <action>update_coordinates_view</action>
        <action>refresh:CUR_DISPLAY</action>
        <action>check_button</action>
        <action>refresh:PREVIEW</action>
      </timer>
      
      <checkbox visible="false">
        <variable>PREVIEW</variable>
        <default>false</default>
'
if [ "$BUT_VAR" = "true" ]; then
  export MAIN=$MAIN'<input>cat $PREV</input>'
else
  export MAIN=$MAIN'<input file>'$PREV'</input>'
fi
MAIN=$MAIN'
        <action>if true draw</action>
        <action>if true refresh:PICTURE</action>
        <action>if true undo4prev</action>
        <action>if true echo 2 > $BUT_STATE</action>
        <action>if true refresh:PREVIEW</action>
      </checkbox>
    </hbox>
  </vbox>
<action signal="show">echo EXIT > $EXITSTATUS</action>
<action signal="hide">exit:abort</action>
</window>
'
# -----------------------------------------------------------------------------

# -------------------
# Display Main window
# -------------------

$GTKDIALOG -cp MAIN

if [ ! -f $EXITSTATUS ]; then
  Xdialog --timeout 10 --title "$APPNAME" --msgbox "Something's missing or doesn't work right.\nDouble check requirements. Exiting..." 0 0 & exit 1
fi
[ "`cat $EXITSTATUS`" = "EXIT" ] && exit 0 || exec $MYNAME

###############################################################################
